from django.dispatch import receiver
from django_aetos.signals import collect_metrics

from .models import Book


@receiver(collect_metrics, dispatch_uid="metric_books_count")
def metric_books_count(sender, **kwargs):
    yield {
        "name": "books_count",
        "help": "Total number of books",
        "type": "counter",
        "value": Book.objects.count(),
    }


@receiver(collect_metrics, dispatch_uid="metric_universes_count")
def metric_universes_count(sender, **kwargs):
    yield {
        "name": "universes_count",
        "help": "Total number of universes",
        "type": "counter",
        "value": "1",
    }



import pytest


@pytest.mark.django_db
def test_e2e(client):
    resp = client.get("/metrics")
    assert (
        resp.content.decode()
        == """# HELP books_count Total number of books
# TYPE books_count counter
books_count 0
# HELP universes_count Total number of universes
# TYPE universes_count counter
universes_count 1
"""
    )



import re

from django.shortcuts import render

from .signals import collect_metrics


def export_metrics(request):
    metrics = []
    known_metrics = set()

    for _, signal_metrics in collect_metrics.send(None):
        for metric in signal_metrics:
            assert metric.keys() == {"name", "help", "type", "value"}
            metric = metric.copy()
            metric["name_without_labels"] = metric["name"].partition("{")[0]
            if metric["name_without_labels"] not in known_metrics:
                metric["add_help"] = True
            known_metrics.add(metric["name_without_labels"])
            metrics.append(metric)

    response = render(
        request,
        "metrics/export.txt",
        context={"metrics": metrics},
        content_type="text/plain",
    )
    response.content = re.sub(b"\n+", b"\n", response.content)
    return response
